@@ -28,19 +28,21 @@ It assumes that live migrations are possible in your cluster.
 """
 
 import copy
+import datetime
 import itertools
 import math
 import random
+import re
 
 import oslo_cache
 from oslo_config import cfg
 from oslo_log import log
 import oslo_utils
 
-from watcher._i18n import _LI, _LW, _
+from watcher._i18n import _
 from watcher.common import exception
-from watcher.decision_engine.cluster.history import ceilometer as \
-    ceilometer_cluster_history
+from watcher.datasource import ceilometer as ceil
+from watcher.datasource import gnocchi as gnoc
 from watcher.decision_engine.model import element
 from watcher.decision_engine.strategy.strategies import base
 
@@ -72,6 +74,7 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
         """
         super(WorkloadStabilization, self).__init__(config, osc)
         self._ceilometer = None
+        self._gnocchi = None
         self._nova = None
         self.weights = None
         self.metrics = None
@@ -79,6 +82,7 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
         self.host_choice = None
         self.instance_metrics = None
         self.retry_count = None
+        self.periods = None
 
     @classmethod
     def get_name(cls):
@@ -92,6 +96,10 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
     def get_translatable_display_name(cls):
         return "Workload stabilization"
 
+    @property
+    def granularity(self):
+        return self.input_parameters.get('granularity', 300)
+
     @classmethod
     def get_schema(cls):
         return {
@@ -119,7 +127,7 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
                     "description": "Mapping to get hardware statistics using"
                                    " instance metrics",
                     "type": "object",
-                    "default": {"cpu_util": "hardware.cpu.util",
+                    "default": {"cpu_util": "compute.node.cpu.percent",
                                 "memory.resident": "hardware.memory.used"}
                 },
                 "host_choice": {
@@ -137,15 +145,41 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
                     "description": "Count of random returned hosts",
                     "type": "number",
                     "default": 1
-                }
+                },
+                "periods": {
+                    "description": "These periods are used to get statistic "
+                                   "aggregation for instance and host "
+                                   "metrics. The period is simply a repeating"
+                                   " interval of time into which the samples"
+                                   " are grouped for aggregation. Watcher "
+                                   "uses only the last period of all recieved"
+                                   " ones.",
+                    "type": "object",
+                    "default": {"instance": 720, "node": 600}
+                },
+                "granularity": {
+                    "description": "The time between two measures in an "
+                                   "aggregated timeseries of a metric.",
+                    "type": "number",
+                    "default": 300
+                },
             }
         }
 
+    @classmethod
+    def get_config_opts(cls):
+        return [
+            cfg.StrOpt(
+                "datasource",
+                help="Data source to use in order to query the needed metrics",
+                default="ceilometer",
+                choices=["ceilometer", "gnocchi"])
+        ]
+
     @property
     def ceilometer(self):
         if self._ceilometer is None:
-            self._ceilometer = (ceilometer_cluster_history.
-                                CeilometerClusterHistory(osc=self.osc))
+            self._ceilometer = ceil.CeilometerHelper(osc=self.osc)
         return self._ceilometer
 
     @property
@@ -162,6 +196,16 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
     def ceilometer(self, c):
         self._ceilometer = c
 
+    @property
+    def gnocchi(self):
+        if self._gnocchi is None:
+            self._gnocchi = gnoc.GnocchiHelper(osc=self.osc)
+        return self._gnocchi
+
+    @gnocchi.setter
+    def gnocchi(self, gnocchi):
+        self._gnocchi = gnocchi
+
     def transform_instance_cpu(self, instance_load, host_vcpus):
         """Transform instance cpu utilization to overall host cpu utilization.
 
@@ -174,30 +218,40 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
                 (instance_load['vcpus'] / float(host_vcpus)))
 
     @MEMOIZE
-    def get_instance_load(self, instance_uuid):
-        """Gathering instance load through ceilometer statistic.
+    def get_instance_load(self, instance):
+        """Gathering instance load through ceilometer/gnocchi statistic.
 
-        :param instance_uuid: instance for which statistic is gathered.
+        :param instance: instance for which statistic is gathered.
         :return: dict
         """
         LOG.debug('get_instance_load started')
-        instance_vcpus = self.compute_model.get_resource_by_uuid(
-            element.ResourceType.cpu_cores).get_capacity(
-                self.compute_model.get_instance_by_uuid(instance_uuid))
-        instance_load = {'uuid': instance_uuid, 'vcpus': instance_vcpus}
+        instance_load = {'uuid': instance.uuid, 'vcpus': instance.vcpus}
         for meter in self.metrics:
-            avg_meter = self.ceilometer.statistic_aggregation(
-                resource_id=instance_uuid,
-                meter_name=meter,
-                period="120",
-                aggregate='min'
-            )
+            avg_meter = None
+            if self.config.datasource == "ceilometer":
+                avg_meter = self.ceilometer.statistic_aggregation(
+                    resource_id=instance.uuid,
+                    meter_name=meter,
+                    period=self.periods['instance'],
+                    aggregate='min'
+                )
+            elif self.config.datasource == "gnocchi":
+                stop_time = datetime.datetime.utcnow()
+                start_time = stop_time - datetime.timedelta(
+                    seconds=int(self.periods['instance']))
+                avg_meter = self.gnocchi.statistic_aggregation(
+                    resource_id=instance.uuid,
+                    metric=meter,
+                    granularity=self.granularity,
+                    start_time=start_time,
+                    stop_time=stop_time,
+                    aggregation='mean'
+                )
             if avg_meter is None:
                 LOG.warning(
-                    _LW("No values returned by %(resource_id)s "
-                        "for %(metric_name)s") % dict(
-                        resource_id=instance_uuid,
-                        metric_name=meter))
+                    "No values returned by %(resource_id)s "
+                    "for %(metric_name)s" % dict(
+                        resource_id=instance.uuid, metric_name=meter))
                 avg_meter = 0
             if meter == 'cpu_util':
                 avg_meter /= float(100)
@@ -208,10 +262,8 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
         normalized_hosts = copy.deepcopy(hosts)
         for host in normalized_hosts:
             if 'memory.resident' in normalized_hosts[host]:
-                h_memory = self.compute_model.get_resource_by_uuid(
-                    element.ResourceType.memory).get_capacity(
-                        self.compute_model.get_node_by_uuid(host))
-                normalized_hosts[host]['memory.resident'] /= float(h_memory)
+                node = self.compute_model.get_node_by_uuid(host)
+                normalized_hosts[host]['memory.resident'] /= float(node.memory)
 
         return normalized_hosts
 
@@ -224,27 +276,44 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
     def get_hosts_load(self):
         """Get load of every available host by gathering instances load"""
         hosts_load = {}
-        for node_id in self.get_available_nodes():
+        for node_id, node in self.get_available_nodes().items():
             hosts_load[node_id] = {}
-            host_vcpus = self.compute_model.get_resource_by_uuid(
-                element.ResourceType.cpu_cores).get_capacity(
-                    self.compute_model.get_node_by_uuid(node_id))
-            hosts_load[node_id]['vcpus'] = host_vcpus
-
+            hosts_load[node_id]['vcpus'] = node.vcpus
             for metric in self.metrics:
-                avg_meter = self.ceilometer.statistic_aggregation(
-                    resource_id=node_id,
-                    meter_name=self.instance_metrics[metric],
-                    period="60",
-                    aggregate='avg'
-                )
+                resource_id = ''
+                avg_meter = None
+                meter_name = self.instance_metrics[metric]
+                if re.match('^compute.node', meter_name) is not None:
+                    resource_id = "%s_%s" % (node.uuid, node.hostname)
+                else:
+                    resource_id = node_id
+                if self.config.datasource == "ceilometer":
+                    avg_meter = self.ceilometer.statistic_aggregation(
+                        resource_id=resource_id,
+                        meter_name=self.instance_metrics[metric],
+                        period=self.periods['node'],
+                        aggregate='avg'
+                    )
+                elif self.config.datasource == "gnocchi":
+                    stop_time = datetime.datetime.utcnow()
+                    start_time = stop_time - datetime.timedelta(
+                        seconds=int(self.periods['node']))
+                    avg_meter = self.gnocchi.statistic_aggregation(
+                        resource_id=resource_id,
+                        metric=self.instance_metrics[metric],
+                        granularity=self.granularity,
+                        start_time=start_time,
+                        stop_time=stop_time,
+                        aggregation='mean'
+                    )
+
                 if avg_meter is None:
                     raise exception.NoSuchMetricForHost(
-                        metric=self.instance_metrics[metric],
+                        metric=meter_name,
                         host=node_id)
-                if self.instance_metrics[metric] == 'hardware.memory.used':
+                if meter_name == 'hardware.memory.used':
                     avg_meter /= oslo_utils.units.Ki
-                if self.instance_metrics[metric] == 'hardware.cpu.util':
+                if meter_name == 'compute.node.cpu.percent':
                     avg_meter /= 100
                 hosts_load[node_id][metric] = avg_meter
         return hosts_load
@@ -275,34 +344,31 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
                       " for %s in weight dict.") % metric)
         return weighted_sd
 
-    def calculate_migration_case(self, hosts, instance_id,
-                                 src_node_id, dst_node_id):
+    def calculate_migration_case(self, hosts, instance, src_node, dst_node):
         """Calculate migration case
 
         Return list of standard deviation values, that appearing in case of
         migration of instance from source host to destination host
         :param hosts: hosts with their workload
-        :param instance_id: the virtual machine
-        :param src_node_id: the source node id
-        :param dst_node_id: the destination node id
+        :param instance: the virtual machine
+        :param src_node: the source node
+        :param dst_node: the destination node
         :return: list of standard deviation values
         """
         migration_case = []
         new_hosts = copy.deepcopy(hosts)
-        instance_load = self.get_instance_load(instance_id)
-        d_host_vcpus = new_hosts[dst_node_id]['vcpus']
-        s_host_vcpus = new_hosts[src_node_id]['vcpus']
+        instance_load = self.get_instance_load(instance)
+        s_host_vcpus = new_hosts[src_node.uuid]['vcpus']
+        d_host_vcpus = new_hosts[dst_node.uuid]['vcpus']
         for metric in self.metrics:
             if metric is 'cpu_util':
-                new_hosts[src_node_id][metric] -= self.transform_instance_cpu(
-                    instance_load,
-                    s_host_vcpus)
-                new_hosts[dst_node_id][metric] += self.transform_instance_cpu(
-                    instance_load,
-                    d_host_vcpus)
+                new_hosts[src_node.uuid][metric] -= (
+                    self.transform_instance_cpu(instance_load, s_host_vcpus))
+                new_hosts[dst_node.uuid][metric] += (
+                    self.transform_instance_cpu(instance_load, d_host_vcpus))
             else:
-                new_hosts[src_node_id][metric] -= instance_load[metric]
-                new_hosts[dst_node_id][metric] += instance_load[metric]
+                new_hosts[src_node.uuid][metric] -= instance_load[metric]
+                new_hosts[dst_node.uuid][metric] += instance_load[metric]
         normalized_hosts = self.normalize_hosts_load(new_hosts)
         for metric in self.metrics:
             migration_case.append(self.get_sd(normalized_hosts, metric))
@@ -324,29 +390,27 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
 
         instance_host_map = []
         nodes = list(self.get_available_nodes())
-        for source_hp_id in nodes:
+        for src_host in nodes:
+            src_node = self.compute_model.get_node_by_uuid(src_host)
             c_nodes = copy.copy(nodes)
-            c_nodes.remove(source_hp_id)
+            c_nodes.remove(src_host)
             node_list = yield_nodes(c_nodes)
-            instances_id = self.compute_model.get_mapping(). \
-                get_node_instances_by_uuid(source_hp_id)
-            for instance_id in instances_id:
+            for instance in self.compute_model.get_node_instances(src_node):
                 min_sd_case = {'value': len(self.metrics)}
-                instance = self.compute_model.get_instance_by_uuid(instance_id)
                 if instance.state not in [element.InstanceState.ACTIVE.value,
                                           element.InstanceState.PAUSED.value]:
                     continue
-                for dst_node_id in next(node_list):
-                    sd_case = self.calculate_migration_case(hosts, instance_id,
-                                                            source_hp_id,
-                                                            dst_node_id)
+                for dst_host in next(node_list):
+                    dst_node = self.compute_model.get_node_by_uuid(dst_host)
+                    sd_case = self.calculate_migration_case(
+                        hosts, instance, src_node, dst_node)
 
                     weighted_sd = self.calculate_weighted_sd(sd_case[:-1])
 
                     if weighted_sd < min_sd_case['value']:
                         min_sd_case = {
-                            'host': dst_node_id, 'value': weighted_sd,
-                            's_host': source_hp_id, 'instance': instance_id}
+                            'host': dst_node.uuid, 'value': weighted_sd,
+                            's_host': src_node.uuid, 'instance': instance.uuid}
                         instance_host_map.append(min_sd_case)
         return sorted(instance_host_map, key=lambda x: x['value'])
 
@@ -394,17 +458,21 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
         return self.solution
 
     def pre_execute(self):
-        LOG.info(_LI("Initializing Workload Stabilization"))
+        LOG.info("Initializing Workload Stabilization")
 
         if not self.compute_model:
             raise exception.ClusterStateNotDefined()
 
+        if self.compute_model.stale:
+            raise exception.ClusterStateStale()
+
         self.weights = self.input_parameters.weights
         self.metrics = self.input_parameters.metrics
         self.thresholds = self.input_parameters.thresholds
         self.host_choice = self.input_parameters.host_choice
         self.instance_metrics = self.input_parameters.instance_metrics
         self.retry_count = self.input_parameters.retry_count
+        self.periods = self.input_parameters.periods
 
     def do_execute(self):
         migration = self.check_threshold()
@@ -413,19 +481,16 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
             min_sd = 1
             balanced = False
             for instance_host in migration:
-                dst_hp_disk = self.compute_model.get_resource_by_uuid(
-                    element.ResourceType.disk).get_capacity(
-                        self.compute_model.get_node_by_uuid(
-                            instance_host['host']))
-                instance_disk = self.compute_model.get_resource_by_uuid(
-                    element.ResourceType.disk).get_capacity(
-                        self.compute_model.get_instance_by_uuid(
-                            instance_host['instance']))
-                if instance_disk > dst_hp_disk:
+                instance = self.compute_model.get_instance_by_uuid(
+                    instance_host['instance'])
+                src_node = self.compute_model.get_node_by_uuid(
+                    instance_host['s_host'])
+                dst_node = self.compute_model.get_node_by_uuid(
+                    instance_host['host'])
+                if instance.disk > dst_node.disk:
                     continue
                 instance_load = self.calculate_migration_case(
-                    hosts_load, instance_host['instance'],
-                    instance_host['s_host'], instance_host['host'])
+                    hosts_load, instance, src_node, dst_node)
                 weighted_sd = self.calculate_weighted_sd(instance_load[:-1])
                 if weighted_sd < min_sd:
                     min_sd = weighted_sd
