@@ -18,6 +18,7 @@
 #
 import collections
 import copy
+import datetime
 import mock
 
 from watcher.applier.loading import default
@@ -26,16 +27,30 @@ from watcher.common import exception
 from watcher.decision_engine.model import model_root
 from watcher.decision_engine.strategy import strategies
 from watcher.tests import base
+from watcher.tests.decision_engine.model import ceilometer_metrics
 from watcher.tests.decision_engine.model import faker_cluster_state
-from watcher.tests.decision_engine.model import faker_metrics_collector
+from watcher.tests.decision_engine.model import gnocchi_metrics
+from watcher.tests.decision_engine.model import monasca_metrics
 
 
 class TestBasicConsolidation(base.TestCase):
 
+    scenarios = [
+        ("Ceilometer",
+         {"datasource": "ceilometer",
+          "fake_datasource_cls": ceilometer_metrics.FakeCeilometerMetrics}),
+        ("Monasca",
+         {"datasource": "monasca",
+          "fake_datasource_cls": monasca_metrics.FakeMonascaMetrics}),
+        ("Gnocchi",
+         {"datasource": "gnocchi",
+          "fake_datasource_cls": gnocchi_metrics.FakeGnocchiMetrics}),
+    ]
+
     def setUp(self):
         super(TestBasicConsolidation, self).setUp()
         # fake metrics
-        self.fake_metrics = faker_metrics_collector.FakerMetricsCollector()
+        self.fake_metrics = self.fake_datasource_cls()
         # fake cluster
         self.fake_cluster = faker_cluster_state.FakerModelCollector()
 
@@ -50,11 +65,11 @@ class TestBasicConsolidation(base.TestCase):
         self.m_model = p_model.start()
         self.addCleanup(p_model.stop)
 
-        p_ceilometer = mock.patch.object(
-            strategies.BasicConsolidation, "ceilometer",
+        p_datasource = mock.patch.object(
+            strategies.BasicConsolidation, self.datasource,
             new_callable=mock.PropertyMock)
-        self.m_ceilometer = p_ceilometer.start()
-        self.addCleanup(p_ceilometer.stop)
+        self.m_datasource = p_datasource.start()
+        self.addCleanup(p_datasource.stop)
 
         p_audit_scope = mock.patch.object(
             strategies.BasicConsolidation, "audit_scope",
@@ -66,9 +81,10 @@ class TestBasicConsolidation(base.TestCase):
         self.m_audit_scope.return_value = mock.Mock()
 
         self.m_model.return_value = model_root.ModelRoot()
-        self.m_ceilometer.return_value = mock.Mock(
+        self.m_datasource.return_value = mock.Mock(
             statistic_aggregation=self.fake_metrics.mock_get_statistics)
-        self.strategy = strategies.BasicConsolidation(config=mock.Mock())
+        self.strategy = strategies.BasicConsolidation(
+            config=mock.Mock(datasource=self.datasource))
 
     def test_cluster_size(self):
         size_cluster = len(
@@ -126,7 +142,7 @@ class TestBasicConsolidation(base.TestCase):
         instance_0_score = 0.023333333333333355
         self.assertEqual(
             instance_0_score,
-            self.strategy.calculate_score_instance(instance_0, ))
+            self.strategy.calculate_score_instance(instance_0))
 
     def test_basic_consolidation_weight(self):
         model = self.fake_cluster.generate_scenario_1()
@@ -142,9 +158,6 @@ class TestBasicConsolidation(base.TestCase):
             instance_0_weight_assert,
             self.strategy.calculate_weight(instance_0, cores, disk, mem))
 
-    def test_calculate_migration_efficacy(self):
-        self.strategy.calculate_migration_efficacy()
-
     def test_exception_model(self):
         self.m_model.return_value = None
         self.assertRaises(
@@ -213,7 +226,7 @@ class TestBasicConsolidation(base.TestCase):
 
         expected_num_migrations = 5
         expected_power_state = 3
-        expected_global_efficacy = 60
+        expected_global_efficacy = 75
 
         num_migrations = actions_counter.get("migrate", 0)
         num_node_state_change = actions_counter.get(
@@ -256,3 +269,69 @@ class TestBasicConsolidation(base.TestCase):
             loaded_action = loader.load(action['action_type'])
             loaded_action.input_parameters = action['input_parameters']
             loaded_action.validate_parameters()
+
+    def test_periods(self):
+        model = self.fake_cluster.generate_scenario_1()
+        self.m_model.return_value = model
+        node_1 = model.get_node_by_uuid("Node_1")
+        p_ceilometer = mock.patch.object(
+            strategies.BasicConsolidation, "ceilometer")
+        m_ceilometer = p_ceilometer.start()
+        self.addCleanup(p_ceilometer.stop)
+        p_monasca = mock.patch.object(strategies.BasicConsolidation, "monasca")
+        m_monasca = p_monasca.start()
+        self.addCleanup(p_monasca.stop)
+        p_gnocchi = mock.patch.object(strategies.BasicConsolidation, "gnocchi")
+        m_gnocchi = p_gnocchi.start()
+        self.addCleanup(p_gnocchi.stop)
+        datetime_patcher = mock.patch.object(
+            datetime, 'datetime',
+            mock.Mock(wraps=datetime.datetime)
+        )
+        mocked_datetime = datetime_patcher.start()
+        mocked_datetime.utcnow.return_value = datetime.datetime(
+            2017, 3, 19, 18, 53, 11, 657417)
+        self.addCleanup(datetime_patcher.stop)
+        m_monasca.return_value = mock.Mock(
+            statistic_aggregation=self.fake_metrics.mock_get_statistics)
+        m_ceilometer.return_value = mock.Mock(
+            statistic_aggregation=self.fake_metrics.mock_get_statistics)
+        m_gnocchi.return_value = mock.Mock(
+            statistic_aggregation=self.fake_metrics.mock_get_statistics)
+        self.strategy.calculate_score_node(node_1)
+        resource_id = "%s_%s" % (node_1.uuid, node_1.hostname)
+        if self.strategy.config.datasource == "ceilometer":
+            m_ceilometer.statistic_aggregation.assert_called_with(
+                aggregate='avg', meter_name='compute.node.cpu.percent',
+                period=7200, resource_id=resource_id)
+        elif self.strategy.config.datasource == "monasca":
+            m_monasca.statistic_aggregation.assert_called_with(
+                aggregate='avg', meter_name='cpu.percent',
+                period=7200, dimensions={'hostname': 'Node_1'})
+        elif self.strategy.config.datasource == "gnocchi":
+            stop_time = datetime.datetime.utcnow()
+            start_time = stop_time - datetime.timedelta(
+                seconds=int('7200'))
+            m_gnocchi.statistic_aggregation.assert_called_with(
+                resource_id=resource_id, metric='compute.node.cpu.percent',
+                granularity=300, start_time=start_time, stop_time=stop_time,
+                aggregation='mean')
+
+        self.strategy.input_parameters.update({"period": 600})
+        self.strategy.calculate_score_node(node_1)
+        if self.strategy.config.datasource == "ceilometer":
+            m_ceilometer.statistic_aggregation.assert_called_with(
+                aggregate='avg', meter_name='compute.node.cpu.percent',
+                period=600, resource_id=resource_id)
+        elif self.strategy.config.datasource == "monasca":
+            m_monasca.statistic_aggregation.assert_called_with(
+                aggregate='avg', meter_name='cpu.percent',
+                period=600, dimensions={'hostname': 'Node_1'})
+        elif self.strategy.config.datasource == "gnocchi":
+            stop_time = datetime.datetime.utcnow()
+            start_time = stop_time - datetime.timedelta(
+                seconds=int('600'))
+            m_gnocchi.statistic_aggregation.assert_called_with(
+                resource_id=resource_id, metric='compute.node.cpu.percent',
+                granularity=300, start_time=start_time, stop_time=stop_time,
+                aggregation='mean')
